class CorpusFileList extends RessourceList {
  static options = {
    item: `
      <tr>
        <td><span class="filename"></span></td>
        <td><span class="author"></span></td>
        <td><span class="title"></span></td>
        <td><span class="publishing_year"></span></td>
        <td class="right-align">
          <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
          <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="download" data-position="top" data-tooltip="View"><i class="material-icons">file_download</i></a>
          <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
        </td>
      </tr>
    `.trim(),
    ressourceMapper: corpusFile => {
      return {
        id: corpusFile.id,
        author: corpusFile.author,
        creationDate: corpusFile.creation_date,
        filename: corpusFile.filename,
        publishingYear: corpusFile.publishing_year,
        title: corpusFile.title
      };
    },
    sortValueName: 'creationDate',
    valueNames: [
      {data: ['id']},
      {data: ['creationDate']},
      'author',
      'filename',
      'publishingYear',
      'title'
    ]
  };


  constructor(listElement, options = {}) {
    super(listElement, {...CorpusFileList.options, ...options});
    this.corpusId = listElement.dataset.corpusId;
  }

  init(user) {
    this._init(user.corpora[this.corpusId].files);
  }

  onclick(event) {
    let action;
    let actionButtonElement;
    let corpusFileElement;
    let corpusFileId;
    let deleteModal;
    let deleteModalElement;
    let tmp;

    corpusFileElement = event.target.closest('tr[data-id]');
    if (corpusFileElement === null) {return;}
    corpusFileId = corpusFileElement.dataset.id;
    actionButtonElement = event.target.closest('.action-button[data-action]');
    action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        tmp = document.createElement('div');
        tmp.innerHTML = `
          <div class="modal">
            <div class="modal-content">
              <h4>Confirm corpus deletion</h4>
              <p>Do you really want to delete the corpus file <b>${app.users[this.userId].corpora[this.corpusId].files[corpusFileId].filename}</b>? It will be permanently deleted!</p>
            </div>
            <div class="modal-footer">
              <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
              <a class="btn modal-close red waves-effect waves-light" href="/corpora/${this.corpusId}/files/${corpusFileId}/delete"><i class="material-icons left">delete</i>Delete</a>
            </div>
          </div>
        `.trim();
        deleteModalElement = document.querySelector('#modals').appendChild(tmp.firstChild);
        deleteModal = M.Modal.init(
          deleteModalElement,
          {
            onCloseEnd: () => {
              deleteModal.destroy();
              deleteModalElement.remove();
            }
          }
        );
        deleteModal.open();
        break;
      case 'download':
        window.location.href = `/corpora/${this.corpusId}/files/${corpusFileId}/download`;
        break;
      case 'view':
        window.location.href = `/corpora/${this.corpusId}/files/${corpusFileId}`;
        break;
      default:
        break;
    }
  }

  usersPatchHandler(patch) {
    let corpusFileId;
    let filteredPatch;
    let match;
    let operation;
    let re;
    let valueName;

    re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)`);
    filteredPatch = patch.filter(operation => re.test(operation.path));
    for (operation of filteredPatch) {
      switch(operation.op) {
        case 'add':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {
            this.add(operation.value);
          }
          break;
        case 'remove':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {
            [match, corpusFileId] = operation.path.match(re);
            this.remove(corpusFileId);
          }
          break;
        case 'replace':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)/(author|filename|publishing_year|title)$`);
          if (re.test(operation.path)) {
            [match, corpusFileId, valueName] = operation.path.match(re);
            this.replace(corpusFileId, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }
}
